// **********************************************************************
//
// Copyright (c) 2003-2007 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#include <Ice/Ice.h>
#include <IceUtil/IceUtil.h>
#include <Freeze/Freeze.h>
#include <StringIntDict.h>
#include <FileInfoDict.h>

using namespace std;
using namespace Demo;
using namespace Freeze;

class TestApp : public Ice::Application
{
public:
    
    TestApp() {}

    virtual int run(int, char*[]);
};

class TestThread : public IceUtil::Thread, public IceUtil::Mutex
{
public:

    TestThread(const Ice::CommunicatorPtr& com, const string& id) :
        _com(com), _id(id), _destroy(false), _count(0)
    {
    }

    virtual void run()
    {
	ConnectionPtr con = Freeze::createConnection(_com, "db");
        StringIntDict sintdict(con, "sint");
        FileInfoDict finfo(con, "finfo");
        for(;;)
        {
            {
                Lock sync(*this);
                if(_destroy)
                {
                    return;
                }
            }
            try
            {
                FileInfoDict::const_iterator p = finfo.find(_id);
                FileInfo info = p->second;
                int found = 0;
                for(StringSeq::const_iterator q = info.locations.begin(); q != info.locations.end(); ++q)
                {
                    if(sintdict.find(*q) == sintdict.end())
                    {
                        cout << "failed to lookup!" << endl;
                    }
                    else
                    {
                        ++found;
                    }
                }
                assert(found == 4);
                ++_count;
            }
            catch(const Freeze::DeadlockException&)
            {
                cout << "deadlock" << endl;
                continue;
            }
            catch(const Freeze::DatabaseException& ex)
            {
                cerr << "error: " << ex << endl;
            }	
        }
    }

    void destroy()
    {
        Lock sync(*this);
        _destroy = true;
    }

    int iterations() const
    {
        return _count;
    }
private:
    const Ice::CommunicatorPtr _com;
    const string _id;
    bool _destroy;
    int _count;
};
typedef IceUtil::Handle<TestThread> TestThreadPtr;

int
TestApp::run(int argc, char* argv[])
{
    ConnectionPtr con = Freeze::createConnection(communicator(), "db");
    StringIntDict sint(con, "sint");
    FileInfoDict finfo(con, "finfo");
    sint.put(StringIntDict::value_type("a", 0));
    sint.put(StringIntDict::value_type("b", 0));
    sint.put(StringIntDict::value_type("c", 0));
    sint.put(StringIntDict::value_type("d", 0));
    FileInfo i;
    i.locations.push_back("a");
    i.locations.push_back("b");
    i.locations.push_back("c");
    i.locations.push_back("d");
    vector<string> ids;
    ids.push_back("a");
    ids.push_back("b");
    ids.push_back("c");
    ids.push_back("d");
    ids.push_back("e");
    vector<string>::const_iterator p;
    for(p = ids.begin(); p != ids.end(); ++p)
    {
        finfo.put(FileInfoDict::value_type(*p, i));
    }
    vector<TestThreadPtr> threads;
    for(p = ids.begin(); p != ids.end(); ++p)
    {
        TestThreadPtr t = new TestThread(communicator(), *p);
        t->start();
        threads.push_back(t);
    }

    sleep(10);

    for(vector<TestThreadPtr>::const_iterator q =  threads.begin(); q != threads.end(); ++q)
    {
        (*q)->destroy();
        (*q)->getThreadControl().join();
        cout << (*q)->iterations() << endl;
    }
    
    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[])
{
    TestApp app;
    return app.main(argc, argv);
}
